unit unadxOutlookForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Outlook2000, adxolFormsManager, StdCtrls, ExtCtrls, ActiveX, ComObj,
  ComCtrls;

type
  TadxOlForm2 = class(TadxOlForm, IDropTarget)
    Panel1: TPanel;
    Label1: TLabel;
    MemoDragTarget: TMemo;
    procedure adxOlFormCreate(Sender: TObject);
    procedure adxOlFormADXDragOverMinimized(const dataObj: IDataObject;
      grfKeyState: Integer; pt: TPoint; var dwEffect: Integer;
      var Expand: LongBool);
  private
    function DragEnter(const dataObj: IDataObject; grfKeyState: Longint;
      pt: TPoint; var dwEffect: Longint): HResult; stdcall;
    function DragOver(grfKeyState: Longint; pt: TPoint;
      var dwEffect: Longint): HResult; stdcall;
    function DragLeave: HResult; stdcall;
    function Drop(const dataObj: IDataObject; grfKeyState: Longint; pt: TPoint;
      var dwEffect: Longint): HResult; stdcall;
    function DoFormats(const dataObj: IDataObject;
      strFormat: string; var FormatEtc: TFormatEtc): Boolean;
    function DoDragOver(const dataObj: IDataObject): Boolean;
  protected
    { Protected declarations }
  public
    destructor Destroy; override;
  published
    { Published declarations }
  end;

{NOTE: The adxOlForm2 variable is intended for the exclusive use
       by the TadxOlFormsCollectionItem Designer.
       NEVER use this variable for other purposes.}
var
  adxOlForm2 : TadxOlForm2;

implementation

{$R *.DFM}

{ TadxOlForm2 }

destructor TadxOlForm2.Destroy;
begin
  RevokeDragDrop(Handle);
  inherited;
end;

procedure TadxOlForm2.adxOlFormCreate(Sender: TObject);
begin
  RegisterDragDrop(Handle, Self);
  //RegisterDragDrop(MemoDragTarget.Handle, Self);
end;

procedure TadxOlForm2.adxOlFormADXDragOverMinimized(
  const dataObj: IDataObject; grfKeyState: Integer; pt: TPoint;
  var dwEffect: Integer; var Expand: LongBool);
begin
  Expand := false;
  if DoDragOver(dataObj) then
    Expand := true;
end;

function TadxOlForm2.DragEnter(const dataObj: IDataObject;
  grfKeyState: Integer; pt: TPoint; var dwEffect: Integer): HResult;
begin
  dwEffect := DROPEFFECT_NONE;
  if DoDragOver(dataObj) then begin
    dwEffect := DROPEFFECT_COPY;
    Result := S_OK;
  end;
end;

function TadxOlForm2.DragOver(grfKeyState: Integer; pt: TPoint;
  var dwEffect: Integer): HResult;
begin
  Result := S_OK;
end;

function TadxOlForm2.Drop(const dataObj: IDataObject; grfKeyState: Integer;
  pt: TPoint; var dwEffect: Integer): HResult;
var
  sel: Selection;
  mlItem: _MailItem;
  TempFormat: TFormatEtc;
  TempMedium: TStgMedium;
begin
  MemoDragTarget.Lines.Clear;
  mlItem := nil;
  if DoFormats(dataObj, 'RenPrivateItem', TempFormat) then begin
    sel := ExplorerObj.Selection;
    if Assigned(sel) then begin
      try
        if sel.Item(1).QueryInterface(IID__MailItem, mlItem) = S_OK then begin
          if Assigned(mlItem) then begin
            try
              MemoDragTarget.Lines.Add(mlItem.Body);
              Result := S_OK;
            finally
              mlItem := nil;
            end;
          end;
        end;
      finally
        sel := nil;
      end;
    end;
  end else begin
    if DoFormats(dataObj, 'Rich Text Format', TempFormat) then begin
      if dataObj.QueryGetData(TempFormat) = S_OK then begin
        if dataObj.GetData(TempFormat, TempMedium) = S_OK then begin
          MemoDragTarget.Lines.Add(PChar(GlobalLock(TempMedium.hGlobal)));
          GlobalUnLock(TempMedium.hglobal);
          Result := S_OK;
        end;
      end;
    end;
  end;
end;

function TadxOlForm2.DragLeave: HResult;
begin
  Result := S_OK;
end;

function TadxOlForm2.DoDragOver(const dataObj: IDataObject): Boolean;
var
  sel: Selection;
  mlItem: _MailItem;
  format: TFormatEtc;
begin
  Result := false;
  mlItem := nil;
  if DoFormats(dataObj, 'RenPrivateItem', format) then begin
    sel := ExplorerObj.Selection;
    if Assigned(sel) then begin
      try
        if sel.Count = 1 then begin
          if sel.Item(1).QueryInterface(IID__MailItem, mlItem) = S_OK then begin
            if Assigned(mlItem) then begin
              try
                Result := true;
              finally
                mlItem := nil;
              end;
            end;
          end;
        end;
      finally
        sel := nil;
      end;
    end;
  end else begin
    if DoFormats(dataObj, 'Rich Text Format', format) then begin
      Result := true;
    end;
  end;
end;

function TadxOlForm2.DoFormats(const dataObj: IDataObject;
  strFormat: string; var FormatEtc: TFormatEtc): Boolean;
var
  Formats: IEnumFORMATETC;
  TempFormat: TFormatEtc;
  pFormatName: PChar;
  count: Integer;
begin
  Result := False;
  if dataObj.EnumFormatEtc(DATADIR_GET, Formats) = S_OK then begin
    while (Formats.Next(1,TempFormat,@count)=S_OK) and (count>0) do begin
      try
        pFormatName := AllocMem(255);
        GetClipboardFormatName(TempFormat.cfFormat, pFormatName, 254);
        if (string(pFormatName) = strFormat) then begin
          FormatEtc := TempFormat;
          Result := True;
          Exit;
        end;
      finally
        FreeMem(pFormatName);
      end;
    end;
  end;
end;

initialization
  RegisterClass(TPersistentClass(TadxOlForm2));

finalization
end.
