unit unadxWordTaskPane;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Word2000, adxwdFormsManager, StdCtrls, ExtCtrls, ActiveX, ComObj,
  ComCtrls;

type
  TadxWordTaskPane1 = class(TadxWordTaskPane, IDropTarget)
    Panel1: TPanel;
    Label1: TLabel;
    MemoDragTarget: TMemo;
    procedure adxWordTaskPaneCreate(Sender: TObject);
    procedure adxWordTaskPaneADXDragOverMinimized(
      const dataObj: IDataObject; grfKeyState: Integer; pt: TPoint;
      var dwEffect: Integer; var Expand: LongBool);
  private
    function DragEnter(const dataObj: IDataObject; grfKeyState: Longint;
      pt: TPoint; var dwEffect: Longint): HResult; stdcall;
    function DragOver(grfKeyState: Longint; pt: TPoint;
      var dwEffect: Longint): HResult; stdcall;
    function DragLeave: HResult; stdcall;
    function Drop(const dataObj: IDataObject; grfKeyState: Longint; pt: TPoint;
      var dwEffect: Longint): HResult; stdcall;
    function DoFormats(const dataObj: IDataObject; strFormat: string;
      var FormatEtc: TFormatEtc): Boolean;
  protected
    { Protected declarations }
  public
    destructor Destroy; override;
  published
    { Published declarations }
  end;

{NOTE: The adxWordTaskPane1 variable is intended for the exclusive use
       by the TadxWordTaskPanesCollectionItem Designer.
       NEVER use this variable for other purposes.}

var
  adxWordTaskPane1 : TadxWordTaskPane1;

implementation

{$R *.DFM}

{ TadxWordTaskPane1 }

destructor TadxWordTaskPane1.Destroy;
begin
  RevokeDragDrop(handle);
  inherited;
end;

procedure TadxWordTaskPane1.adxWordTaskPaneCreate(Sender: TObject);
begin
  //RegisterDragDrop(MemoDragTarget.Handle, Self);
  RegisterDragDrop(Handle, Self);
end;

procedure TadxWordTaskPane1.adxWordTaskPaneADXDragOverMinimized(
  const dataObj: IDataObject; grfKeyState: Integer; pt: TPoint;
  var dwEffect: Integer; var Expand: LongBool);
var
  format: TFormatEtc;
begin
  Expand := false;
  if DoFormats(dataObj, 'Rich Text Format', format) or
    DoFormats(dataObj, 'RenPrivateItem', format) then  Expand := true;
end;

function TadxWordTaskPane1.DragEnter(const dataObj: IDataObject;
  grfKeyState: Integer; pt: TPoint; var dwEffect: Integer): HResult;
var
  format: TFormatEtc;
begin
  dwEffect := DROPEFFECT_NONE;
  if DoFormats(dataObj, 'Rich Text Format', format) or
    DoFormats(dataObj, 'RenPrivateItem', format) then begin

    dwEffect := DROPEFFECT_COPY;
    Result := S_OK;
  end;
end;

function TadxWordTaskPane1.DragOver(grfKeyState: Integer; pt: TPoint;
  var dwEffect: Integer): HResult;
begin
  Result := S_OK;
  dwEffect := DROPEFFECT_COPY;
end;

function TadxWordTaskPane1.Drop(const dataObj: IDataObject;
  grfKeyState: Integer; pt: TPoint; var dwEffect: Integer): HResult;
var
  TempFormat: TFormatEtc;
  TempMedium: TStgMedium;
begin
  MemoDragTarget.Lines.Clear;
  if DoFormats(dataObj, 'Rich Text Format', TempFormat) or
    DoFormats(dataObj, 'RenPrivateItem', TempFormat) then begin

    TempFormat.cfFormat := CF_TEXT;
    TempFormat.ptd := nil;
    TempFormat.dwAspect := DVASPECT_CONTENT;
    TempFormat.lindex := -1;
    TempFormat.tymed := TYMED_HGLOBAL;

    if dataObj.QueryGetData(TempFormat) = S_OK then begin
      if dataObj.GetData(TempFormat, TempMedium) = S_OK then begin
        MemoDragTarget.Lines.Add(PChar(GlobalLock(TempMedium.hGlobal)));
        GlobalUnLock(TempMedium.hglobal);
        Result := S_OK;
      end;
    end;
  end;
end;

function TadxWordTaskPane1.DragLeave: HResult;
begin
  Result := S_OK;
end;

function TadxWordTaskPane1.DoFormats(const dataObj: IDataObject;
  strFormat: string; var FormatEtc: TFormatEtc): Boolean;
var
  Formats: IEnumFORMATETC;
  TempFormat: TFormatEtc;
  pFormatName: PChar;
  count: Integer;
begin
  Result := False;
  if dataObj.EnumFormatEtc(DATADIR_GET, Formats) = S_OK then begin
    while (Formats.Next(1,TempFormat,@count)=S_OK) and (count>0) do begin
      try
        pFormatName := AllocMem(255);
        GetClipboardFormatName(TempFormat.cfFormat, pFormatName, 254);
        if (string(pFormatName) = strFormat) then begin
          FormatEtc := TempFormat;
          Result := True;
          Exit;
        end;
      finally
        FreeMem(pFormatName);
      end;
    end;
  end;
end;

initialization
  RegisterClass(TPersistentClass(TadxWordTaskPane1));

finalization
end.
